@@ -17,10 +17,12 @@
 	 tweak_info_line/0,tweak_magnet_help/0,statusbar/0]).
 
 -export([toggle_draw/1,point_center/3]).
--export([update_dlist/3,draw/4,get_data/3]).
+-export([update_dlist/3,draw/5,get_data/3]).
 
 -export([tweak_keys/0, menu/0, tweak_magnet_menu/0, constraints_menu/0]).  %% For wings_tweak_win only
 
+-export_type([drag/0]).
+
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 
@@ -28,7 +30,7 @@
 -define(R_ALT, 308).
 
 -include("wings.hrl").
--include("e3d.hrl").
+-include_lib("wings/e3d/e3d.hrl").
 
 -import(lists,[member/2,foldl/3]).
 
@@ -62,6 +64,7 @@
 	 vs,        % [{V,Pos,Distance,Influence}]
 	 vtab=[]}). % [{V,Pos}] (latest)
 
+-type drag() :: #drag{}.
 
 %%%
 %%% Set Default Tweak prefs
@@ -382,7 +385,7 @@ enter_tweak_handler(Ev, What, St, #tweak{id={Action,_},st=#st{sel=Sel}=St0}=T) -
 	  end,
     begin_drag(What, St1, T),
     do_tweak_0(0, 0, 0, 0, {move,screen}),
-    handle_tweak_drag_event_0(Ev,T).
+    {replace,fun(Event) -> handle_tweak_drag_event_0(Event, T) end, Ev}.
 
 %%%
 %%% Tweak Event Handlers
@@ -2423,7 +2426,7 @@ edge_fun(EdList, ClBin, Vtab) ->
     gl:bufferData(?GL_ARRAY_BUFFER, byte_size(ClBin), ClBin, ?GL_STATIC_DRAW),
     gl:bindBuffer(?GL_ARRAY_BUFFER, 0),
     N = byte_size(EdBin) div 12,
-    D = fun() ->
+    D = fun(RS) ->
 		gl:depthFunc(?GL_LEQUAL),
 		gl:bindBuffer(?GL_ARRAY_BUFFER, VboEs),
 		gl:vertexPointer(3, ?GL_FLOAT, 0, 0),
@@ -2435,7 +2438,8 @@ edge_fun(EdList, ClBin, Vtab) ->
 		gl:drawArrays(?GL_LINES, 0, N),
 		gl:disableClientState(?GL_VERTEX_ARRAY),
 		gl:disableClientState(?GL_COLOR_ARRAY),
-		gl:depthFunc(?GL_LESS)
+		gl:depthFunc(?GL_LESS),
+                RS
 	end,
     {call,D,[{vbo,VboEs},{vbo,VboCl}]}.
 
@@ -2463,10 +2467,10 @@ get_data(update_dlist, Data, Acc) ->  % for draw lists
     end.
 
 %% It'll use the list prepared by 'update_dlist' function and then draw it (only for plain draw)
-draw(plain, EdgeList, _D, SelMode) ->
+draw(plain, EdgeList, _D, SelMode, RS) ->
     gl:lineWidth(edge_width(SelMode)),
-    wings_dl:call(EdgeList);
-draw(_,_,_,_) -> ok.
+    wings_dl:call(EdgeList, RS);
+draw(_,_,_,_, RS) -> RS.
 
 edge_width(edge) -> wings_pref:get_value(edge_width);
 edge_width(_) -> 1.
